/*
 * Project name:
     RTC_Read (Demonstration on working with the RTC Module and Software I2C routines)
 * Copyright:
     (c) Mikroelektronika, 2010.
 * Revision History:
     20071210:
       - initial release;
     20101130:
       - adapted for PRO compilers (TL);
 * Description:
     This project is simple demonstration how to read date and time from
     Dallas(Maxim Integrated Products) DS1307 RTC2 (real-time clock).
     Date and time are read from the RTC2 every 1 second and printed on LCD.
 * Test configuration:
     MCU:             AT89S8253
                      http://www.atmel.com/dyn/resources/prod_documents/doc3286.pdf
     Dev.Board:       Easy8051 v6
                      http://www.mikroe.com/eng/products/view/329/easy8051-v6-development-system/
     Oscillator:      External Clock - 10.0000 MHz
     Ext. Modules:    RTC2 extra board on PORT1
                      http://www.mikroe.com/eng/products/view/197/rtc2-board/
     SW:              mikroC for 8051 v2.2
                      http://www.mikroe.com/eng/products/view/232/mikroc-pro-for-8051/
 * NOTES:
     - Pull-up I2C communication lines (P1.0 and P1.1)
     - Turn off I2C communication lines (PORT1) LEDs.
     - Turn on LCD backlight switch on development board.
     - In order to use the example, address pin D0 of DS1307 must be set to 0V.
       (on mikroElektronika's RTC extra board this is done by default)
*/

char seconds, minutes, hours, date, month;  // Global date/time variables
unsigned int year;
unsigned int RTCModuleAddress, YearOffset; // RTC chip description variables

//#define PCF8583         // Uncomment this line if you use PCF8583 RTC chip (mE RTC extra board)
#define DS1307          // Uncomment this line if you use DS1307 RTC chip (mE RTC2 extra board)
  
// Software I2C connections
sbit Soft_I2C_Scl at P1_0_bit;
sbit Soft_I2C_Sda at P1_1_bit;
// End Software I2C connections

// LCD module connections
sbit LCD_RS at P2_0_bit;
sbit LCD_EN at P2_1_bit;

sbit LCD_D4 at P2_2_bit;
sbit LCD_D5 at P2_3_bit;
sbit LCD_D6 at P2_4_bit;
sbit LCD_D7 at P2_5_bit;
// End LCD module connections

//------------------ Performs project-wide init
void Init_Main() {

  #ifdef DS1307
    RTCModuleAddress   = 0xD0;
    YearOffset         = 2000;
  #endif

  Soft_I2C_Init();           // Initialize Soft I2C communication
  Lcd_Init();                // Initialize LCD
  Lcd_Cmd(_LCD_CLEAR);       // Clear LCD display
  Lcd_Cmd(_LCD_CURSOR_OFF);  // Turn cursor off

  LCD_Out(1,1,"Date:");      // Prepare and output static text on LCD
  Lcd_Chr(1,8,':');
  Lcd_Chr(1,11,':');
  LCD_Out(2,1,"Time:");
  Lcd_Chr(2,8,':');
  Lcd_Chr(2,11,':');
}

//--------------------- Reads time and date information from DS1307 RTC
void Read_Time_DS1307() {
  char byte_read;

  Soft_I2C_Start();                   // Issue start signal
  Soft_I2C_Write(RTCModuleAddress);   // RTC module address + write (R#/W = 0)
  Soft_I2C_Write(0);                  // Start from seconds byte
  Soft_I2C_Start();                   // Issue repeated start signal
  Soft_I2C_Write(RTCModuleAddress+1); // RTC module address + read  (R#/W = 1)

  byte_read = Soft_I2C_Read(1);                                // Read seconds byte
  seconds = ((byte_read & 0x70) >> 4)*10 + (byte_read & 0x0F); // Transform seconds

  byte_read = Soft_I2C_Read(1);                                // Read minutes byte
  minutes = ((byte_read & 0x70) >> 4)*10 + (byte_read & 0x0F); // Transform minutes

  byte_read = Soft_I2C_Read(1);                                // Read hours byte
  if (byte_read.B6) {                                          // 12h format
    hours = ((byte_read & 0x10) >> 4)*10 + (byte_read & 0x0F); // Transform hours
    if (byte_read.B5)                                          // PM flag
      hours = hours + 12;
  }
  else
    hours = ((byte_read & 0x30) >> 4)*10 + (byte_read & 0x0F); // Transform hours

  byte_read = Soft_I2C_Read(1);                                // Read weekday byte
  
  byte_read = Soft_I2C_Read(1);                                // Read date byte
  date = ((byte_read & 0x30) >> 4)*10 + (byte_read & 0x0F);    // Transform date
  
  byte_read = Soft_I2C_Read(1);                                // Read month byte
  month = ((byte_read & 0x10) >> 4)*10 + (byte_read & 0x0F);   // Transform month
  
  byte_read = Soft_I2C_Read(1);                                // Read year byte
  year = YearOffset + ((byte_read & 0xF0) >> 4)*10 + (byte_read & 0x0F); // Transform year

  Soft_I2C_Stop();                    // Issue stop signal
}

//--------------------- Reads time and date information from RTC
void Read_Time() {
  #ifdef DS1307
    Read_Time_DS1307();
  #endif
}

//-------------------- Output values to LCD
void Display_Time() {

  Lcd_Chr(1, 6, (date / 10)   + 48);    // Print tens digit of date variable
  Lcd_Chr(1, 7, (date % 10)   + 48);    // Print oness digit of date variable
  Lcd_Chr(1, 9, (month / 10)  + 48);
  Lcd_Chr(1,10, (month % 10)  + 48);
  Lcd_Chr(1,12, ((year / 1000) % 10) + 48);    // Print year
  Lcd_Chr(1,13, ((year / 100)  % 10) + 48);
  Lcd_Chr(1,14, ((year / 10)   % 10) + 48);
  Lcd_Chr(1,15, (year % 10)          + 48);

  Lcd_Chr(2, 6, (hours / 10)   + 48);
  Lcd_Chr(2, 7, (hours % 10)   + 48);
  Lcd_Chr(2, 9, (minutes / 10) + 48);
  Lcd_Chr(2,10, (minutes % 10) + 48);
  Lcd_Chr(2,12, (seconds / 10) + 48);
  Lcd_Chr(2,13, (seconds % 10) + 48);
}

//----------------- Main procedure
void main() {

  Delay_ms(2000);

  Init_Main();               // Perform initialization

  while (1) {                // Endless loop
    Read_Time();             // Read time from RTC
    Display_Time();          // Prepare and display on LCD

    Delay_ms(1000);          // Wait 1 second
  }
}
